#include <stdio.h>

#include "common.h"
#include "gps/library.h"
#include "gps/gps.h"

char library_version[GPS_MAXIMUM_LIBRARY_STRING_LENGTH]={0};

enum firmware_function{
	GPS_GET_LOCATION = 1,
	GPS_GET_SATELLITE_INFO,
	GPS_GET_ANTENNA_STATUS,
	GPS_GET_UTC_TIME,
	GPS_START_NMEA,
	GPS_MAX_FUNCTION
};

static const char *function_name[] = {
	"Get Location Information",
	"Get Satellite Information",
	"Get Antenna Status",
	"Get UTC Time",
	"Start callback NMEA code"
};

static const int function_supported[] = {
	GPS_GET_LOCATION,
	GPS_GET_SATELLITE_INFO,
	GPS_GET_ANTENNA_STATUS,
	GPS_GET_UTC_TIME,
	GPS_START_NMEA,
	GPS_MAX_FUNCTION
};

static void show_welcome_title(void)
{
	printf("GPS SDK Sample -- library version:%s\n", library_version);
	printf(">> Main Menu\n\n");
}

static void show_menu(void)
{
	printf("0) exit\n");
	
	for (int i = 0; i < GPS_MAX_FUNCTION - 1; i++)
	{
		if (function_supported[i] >= 0)
			printf("%d) %s\n", i + 1, function_name[i]);
		else
			break;
	}
	
	printf("\nEnter your choice: ");
}

int get_gps_fix_status()
{
	mrm_err result;
	
	gps_fix_status status;
	
	if( (result = gps_get_fix_status(&status)) != MRM_ERR_NO_ERROR )
	{
		printf("gps_get_fix_status fail. error code=0x%08x\n", result);
		return -1;
	}
	
	if( status == GPS_STATUS_NO_FIX)
	{
		printf("GPS no fixed.\n");
		return -1;
	}
	
	return 0;
}

void get_location_info()
{
	mrm_err result;
	
	if(get_gps_fix_status() < 0)
		return;

	gps_location_t location_info;

	if( (result = gps_get_location(&location_info)) != MRM_ERR_NO_ERROR )
	{
		printf("gps_get_location fail. error code=0x%08x\n", result);
		return;
	}
		
	printf("The latitude is : %.5f degresss.\n", location_info.latitude);
	printf("The longitude is : %.5f degress.\n", location_info.longitude);
	printf("The altitude is : %.2f meters.\n", location_info.altitude);
}

void get_satellite_info()
{
	mrm_err result;
		
	if(get_gps_fix_status() < 0)
		return;

	int sv;
	
	if( (result = gps_get_satellite_in_view(&sv)) != MRM_ERR_NO_ERROR )
	{
		printf("gps_get_satellite_in_view fail. error code=0x%08x\n", result);
		return;
	}
	
	if(sv==0)
	{
		printf("No satellite in view\n");
		return;
	}
	
	gps_satellite_t satellites[32];

	if( (result = gps_get_satellites(satellites)) != MRM_ERR_NO_ERROR )
	{
		printf("gps_get_satellites fail. error code=0x%08x\n", result);
		return;
	}
	
	for(int i=0; i<sv;i++)
	{
		printf("PRN=%d, Elevation=%d Azimuth=%d, SNR=%d.\n",
			satellites[i].prn,
			satellites[i].elevation,
			satellites[i].azimuth,
			satellites[i].snr
		);
	}
}

void get_antenna_status()
{
	mrm_err result;
	
	gps_antenna_status status;

	if( (result = gps_get_antenna_status(&status)) != MRM_ERR_NO_ERROR )
	{
		printf("gps_get_antenna_status fail. error code=0x%08x\n", result);
		return;
	}

	if(status == GPS_ANTENNA_OK)
		printf("Antenna is: %s\n", "OK");
	else if(status == GPS_ANTENNA_OPEN)
		printf("Antenna is: %s\n", "OPEN");
	else if(status == GPS_ANTENNA_SHORT)
		printf("Antenna is: %s\n", "SHORT");
	else
		printf("Antenna is: %s\n", "UNKNOWN");
}

void get_utc_time()
{
	mrm_err result;
	
	if(get_gps_fix_status() < 0)
		return;

	gps_time_t utc_time;

	if( (result = gps_get_utc_time(&utc_time)) != MRM_ERR_NO_ERROR )
	{
		printf("gps_get_utc_time fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("The last UTC Time is : %u/%u/%hu -- %u:%u:%u\n",
	(unsigned)utc_time.day,
	(unsigned)utc_time.month,
	utc_time.year,
	(unsigned)utc_time.hour,
	(unsigned)utc_time.minute,
	(unsigned)utc_time.second);
}

void nmea_pushed(char *NMEA , int length)
{
	printf("%s", NMEA);
}

void nmea_callback_register()
{
	mrm_err result;
	
	if( (result = gps_set_nmea_callback(&nmea_pushed)) != MRM_ERR_NO_ERROR )
	{
		printf("gps_set_nmea_callback fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Press Any key to Stop\n");
	
	WAIT_ENTER();
	
	if( (result = gps_clear_nmea_callback()) != MRM_ERR_NO_ERROR )
	{
		printf("gps_clear_nmea_callback fail. error code=0x%08x\n", result);
		return;
	}
}

int main(int argc, char *argv[])
{
	mrm_err result;
	int op;
	
	char path[256] = "/dev/ttyS2";
	if(argc > 1)
	{
		sprintf(path, "%s", argv[1]);
	}	
	printf("Open path:%s\n", path);

	if( (result = gps_init(path)) != MRM_ERR_NO_ERROR )
	{
		printf("gps_init fail. error code=0x%08x\n", result);
		printf("\nUsage: %s [path...]\r\n", argv[0]);
		printf("\nPress ENTER to continue.\r\n");		
		WAIT_ENTER();
		return -1;
	}
	
	if( (result = gps_get_version(library_version)) != MRM_ERR_NO_ERROR )
	{
		printf("gps_get_version fail. error code=0x%08x\n", result);
		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();
		return -1;
	}

	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		show_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;

		switch(op)
		{
			case GPS_GET_LOCATION:
				get_location_info();
				break;
			case GPS_GET_SATELLITE_INFO:
				get_satellite_info();
				break;
			case GPS_GET_ANTENNA_STATUS:
				get_antenna_status();
				break;
			case GPS_GET_UTC_TIME:
				get_utc_time();
				break;
			case GPS_START_NMEA:
				nmea_callback_register();
				break;
			default:
				printf("Unknown choice!\n");
				break;
		}
		
		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();
	}

	if( (result = gps_deinit()) != MRM_ERR_NO_ERROR )
	{
		printf("gps_deinit. error code=0x%08x\n", result);
		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();
		return -1;
	}

	return 0;
}


