#include <stdio.h>

#include "common.h"
#include "vcil/vcil.h"
#include "string.h"

#if defined (__linux__)
#include <pthread.h>
#endif

#include <unistd.h>
static pthread_t can_thread_handle = 0;
static int thread_enable = 0;
static OSEVENT can_rx_event;

char library_version[VCIL_MAXIMUM_LIBRARY_STRING_LENGTH]={0};
char fw_version[VCIL_MAXIMUM_FIRMWARE_VERSION_LENGTH]={0};

enum {
	FN_ENGINE_TEMPERATURE = 1,
	FN_ENGINE_TOTAL_FUEL_USED,
	FN_ENGINE_SPEED,
	FN_ACCELERATOR_PEDAL_POSITION,
	FN_VEHICLE_SPEED,
	FN_REQUEST_PGN,
	FN_NUMBER
};

static const char *fn_name[FN_NUMBER-1] = {
	"Get Engine Temperature 1",
	"Get Engine total fuel used",
	"Get Engine speed",
	"Get Accelerator pedal position",
	"Get Vehicle Speed",
	"Request PGN"
};

static const int function_list[FN_NUMBER-1] = {
	FN_ENGINE_TEMPERATURE,
	FN_ENGINE_TOTAL_FUEL_USED,
	FN_ENGINE_SPEED,
	FN_ACCELERATOR_PEDAL_POSITION,
	FN_VEHICLE_SPEED,
	FN_REQUEST_PGN
};

int pgn_et1_recv = 0;
int engine_coolant_temperature = 0;
int engine_fuel_temperature1 = 0;

int pgn_lfc_recv = 0;
int engine_total_fuel_used = 0;

int pgn_eec1_recv = 0;
int engine_speed = 0;

int pgn_eec2_recv = 0;
int accelerator_pedal_position = 0;

int pgn_tco1_recv = 0;
float vehicle_speed = 0;

int dtc_on = 0;
int dtc_SPN = 0;
int dtc_FMI = 0;
int dtc_OC = 0;
int dtc_CM = 0;

void check_j1939_data(vcil_j1939_message_t *msg)
{
	switch(msg->pgn)
	{
	
	// PGN 65262 Engine Temperature 1 - ET1 
	case 0xFEEE:
	{
		// Position Lengh    Paramter Name                            SPN
		// 1        1 byte   Engine Coolant Temperature               110 
		// 2        1 byte   Engine Fuel Temperature 1                174 
		// 3-4      2 bytes  Engine Oil Temperature 1                 175 
		// 5-6      2 bytes  Engine Turbocharger Oil Temperature      176 
		// 7        1 byte   Engine Intercooler Temperature           52 
		// 8        1 byte   Engine Intercooler Thermostat Opening    1134
		
		pgn_et1_recv++;
		//1 deg C/bit, -40 deg C offset 
		engine_coolant_temperature = msg->data[0]-40;
		engine_fuel_temperature1 = msg->data[1]-40;
	}
	break;
	
	// PGN 65257 Fuel Consumption (Liquid) – LFC 
	case 0xFEE9:
	{
		// Position Lengh    Paramter Name                            SPN
		// 1-4      4 bytes  Engine Trip Fuel                         182 
		// 5-8      4 bytes  Engine Total Fuel Used                   250 
		
		pgn_lfc_recv++;
		// 0.5L/bits
		engine_total_fuel_used = (msg->data[7] << 24 | msg->data[6]<<16 | msg->data[5]<<8 | msg->data[4])/2;
	}
	break;
	
	// PGN 61444 Electronic Engine Controller 1 - EEC1 
	case 0xF004:
	{
		// Position Lengh    Paramter Name                            SPN
		// 1        4 bits   Engine Torque Mode                       899 
		// 2        1 byte   Driver's Demand Engine - Percent Torque  512 
		// 3        1 byte   Actual Engine - Percent Torque           513 
		// 4-5      2 bytes  Engine Speed                             190 
		// 6        1 byte   Source Address of Controlling device     1483 
		// 7        4 bits   Engine Starter Mode                      1675 
		// 8        1 byte   Engine Demand – Percent Torque           2432 
		pgn_eec1_recv++;
		// 0.125 RPM/bits
		engine_speed = (msg->data[4] << 8 | msg->data[3])/8;
	}
	break;
	
	// PGN 61443 Electronic Engine Controller 2 - EEC2 
	case 0xF003:
	{
		// Position Lengh    Paramter Name                            SPN
		//1.1       2 bits   Accelerator Pedal 1 Low Idle Switch      558 
		//1.3       2 bits   Accelerator Pedal Kickdown Switch        559 
		//1.5       2 bits   Road Speed Limit Status                  1437 
		//1.7       2 bits   Accelerator Pedal 2 Low Idle Switch      2970 
		//2         1 byte   Accelerator Pedal Position 1             91 
		//3         1 byte   Engine Percent Load At Current Speed     92 
		//4         1 byte   Remote Accelerator Pedal Position        974 
		//5         1 byte   Accelerator Pedal Position 2             29 
		//6.1       2 bits   Vehicle Acceleration Rate Limit Status   2979 
		//7         1 byte   Actual Maximum Available - Percent Torque 3357 
		pgn_eec2_recv++;
		accelerator_pedal_position = 0.4*msg->data[1];
	}
	break;
	
	// PGN 65132 Tachograph - TCO1 
	case 0xFE6C:
	{
		// Position Lengh    Paramter Name                            SPN
		//1.1       3 bits   Driver 1 working state                   1612 
		//1.4       3 bits   Driver 2 working state                   1613 
		//1.7       2 bits   Vehicle motion                           1611 
		//2.1       4 bits   Driver 1 Time Related States             1617 
		//2.5       2 bits   Driver card, driver 1                    1615 
		//2.7       2 bits   Vehicle Overspeed                        1614 
		//3.1       4 bits   Driver 2 Time Related States             1618 
		//3.5       2 bits   Driver card, driver 2                    1616 
		//4.1       2 bits   System event                             1622 
		//4.3       2 bits   Handling information                     1621 
		//4.5       2 bits   Tachograph performance                   1620 
		//4.7       2 bits   Direction indicator                      1619 
		//5-6       2 bytes  Tachograph output shaft speed            1623 
		//7-8       2 bytes  Tachograph vehicle speed                 1624 
		pgn_tco1_recv++;
		// 1/256km/h per bits
		vehicle_speed = (msg->data[7] << 8 | msg->data[6])/256.0;
	}
	break;
	
	// PGN 65226 Active Diagnostic Trouble Codes(DTC) - DM1
	case 0xFECA:
	{
		// Position Lengh    Paramter Name
		//1.8-7     2 bits   Malfunction Indicator Lamp Status  
		//1.6-5     2 bits   Red Stop Lamp Status  
		//1.4-3     2 bits   Amber Warning Lamp Status  
		//1.2-1     2 bits   Protect Lamp Status  
		//2.8-7     2 bits   Flash Malfunction Indicator Lamp  
		//2.6-5     2 bits   Flash Red Stop Lamp  
		//2.4-3     2 bits   Flash Amber Warning Lamp  
		//2.2-1     2 bits   Flash Protect Lamp  
		//3         1 bytes  SPN, 8 least significant bits of SPN  
		//4         1 bytes  SPN, second byte of SPN  (most significant at bit 8) 
		//5.8-6     3 bits   SPN, 3 most significant bits  (most significant at bit 8) 
		//5.5-1     5 bits   FMI (most significant at bit 5)
		//6.8       1 bits   SPN Conversion Method  
		//6.7-1     7 bits   Occurrence Count
		
		// MIL LED On
		if(msg->data[0]>>6)
		{
			int current_dtc_SPN = (msg->data[4]>>5) << 16 | (msg->data[3]<<8) | msg->data[2];
			dtc_FMI = msg->data[4]&0x1F;
			dtc_OC = msg->data[5]&0x7F;
			dtc_CM = msg->data[5]>>7;
			
			if(dtc_SPN != current_dtc_SPN)
			{
				dtc_SPN = current_dtc_SPN;
				printf("MIL ON\r\n");
				printf("SPN:%d(0x%X)\r\n", dtc_SPN, dtc_SPN);
				printf("FMI:0x%X\r\n", dtc_FMI);
				printf("OC:%x\r\n", dtc_OC);
				printf("CM:0x%x\r\n", dtc_CM);
			}
			dtc_on = 1;
		}
	}
	break;
	
	default:
		// ignore
		break;	
	}
}

static void *j1939_read_thread(void *p)
{
	printf("J1939 thread start\n");
	while(thread_enable)
	{
		pthread_mutex_lock(can_rx_event.mutex_handle);
		pthread_cond_wait(can_rx_event.event_handle, can_rx_event.mutex_handle);
		pthread_mutex_unlock(can_rx_event.mutex_handle);

		mrm_err result;
		vcil_j1939_message_t message;
		while( (result = vcil_j1939_read(&message)) == MRM_ERR_NO_ERROR)
		{
			check_j1939_data(&message);
		}

		if( result != MRM_ERR_VCIL_DATA_NOT_READY )
		{
			printf("vcil_j1939_read fail. error code=0x%08x\n", result);
			continue;
		}
	
	}
	return 0;
}

static void show_welcome_title(void)
{
	printf("VCIL SDK Sample -- library version:%s\n", library_version);
	printf("                   firmware version:%s\n", fw_version);
	printf(">> Main Menu\n\n");
}

static void show_menu(void)
{
	printf("0) exit\n");
	
	for (int i = 0; i < FN_NUMBER-1; i++)
	{
		printf("%d) %s\n", i + 1, fn_name[i]);		
	}
	
	printf("\nEnter your choice: ");
}

void get_firmware_version()
{
	mrm_err result;
	
	if( (result = vcil_firmware_get_version(fw_version)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_firmware_get_version fail. error code=0x%08x\n", result);
		return;
	}
}

void request_pgn()
{
	vcil_j1939_message_t message;
	
	// clear
	memset(&message, 0, sizeof(message));
	
	message.port = 0;	
	message.pgn = 0xEA00; // Request PGN
	
	printf("Destination(Global 255):");
	unsigned int d;
	scanf("%u", &d);
	
	WAIT_ENTER();
	
	if( d > 255)
	{
		printf("Error range\n");
		return;
	}
	message.destination = d;
	message.source = 0; // ignore
	message.priority = 6;
	message.length = 8;
	
	printf("PGN:");
	unsigned int pgn;
	scanf("%u", &pgn);
	
	WAIT_ENTER();
	
	if( pgn > 65535)
	{
		printf("Error range\n");
		return;
	}
	
	// 1-3 bytes LSB PGN 
	message.data[0] = pgn&0xFF;
	message.data[1] = (pgn>>8)&0xFF;
	message.data[2] = (pgn>>16)&0xFF;

	mrm_err result;

	if( (result = vcil_j1939_write(&message)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_j1939_write fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
	WAIT_ENTER();
}

void j1939_thread_init()
{
	pthread_condattr_t cond_attr;
	pthread_condattr_init ( &cond_attr );
	can_rx_event.event_handle = new pthread_cond_t();
	pthread_cond_init ( can_rx_event.event_handle, &cond_attr );
	
	pthread_mutexattr_t attr;
	pthread_mutexattr_init (&attr);
	pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_RECURSIVE);
	can_rx_event.mutex_handle = new pthread_mutex_t();
	pthread_mutex_init (can_rx_event.mutex_handle, &attr);

	mrm_err result;
	
	if( (result = vcil_j1939_set_event(&can_rx_event)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_j1939_set_event fail. error code=0x%08x\n", result);
		return;
	}
	
	thread_enable = 1;
	
	pthread_attr_t thread_attr;
	pthread_attr_init(&thread_attr);
	int ret = ::pthread_create(&can_thread_handle, &thread_attr, &j1939_read_thread, NULL);
	pthread_attr_destroy(&thread_attr);
	
	if( ret < 0)
	{
		printf("j1939 thread create fail.");
		return;
	}
}

void set_module_mode()
{
	mrm_err result;

	// Setup Protocol to J1939
	if( (result = vcil_set_mode(VCIL_MODE_J1939, VCIL_MODE_CAN, VCIL_MODE_J1708)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_set_mode fail. error code=0x%08x\n", result);
		return;
	}
	
	// Init the CAN bus baudrate, The standard J1939 is 250kbps
	if( (result = vcil_can_set_speed(0, VCIL_CAN_SPEED_250K)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_can_set_speed fail. error code=0x%08x\n", result);
		return;
	}
}


void j1939_thread_deinit()
{
	thread_enable = 0;
	
	pthread_mutex_lock(can_rx_event.mutex_handle);
	pthread_cond_broadcast(can_rx_event.event_handle);
	pthread_mutex_unlock(can_rx_event.mutex_handle);
}

int main(int argc, char *argv[])
{
	mrm_err result;
	int op;
	
	char path[256] = "/dev/ttyA0";
	if(argc > 1)
	{
		sprintf(path, "%s", argv[1]);
	}	
	printf("Open path:%s\n", path);
	
	if( (result = vcil_init(path)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_init fail. error code=0x%08x\n", result);
		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();
		return -1;
	}
	
	if( (result = vcil_get_version(library_version)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_get_version fail. error code=0x%08x\n", result);
		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();
		return -1;
	}
	
	get_firmware_version();
	
	set_module_mode();
	
	j1939_thread_init();
	
	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		show_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;
			
		switch(op)
		{
			case FN_ENGINE_TEMPERATURE:
				if(pgn_et1_recv)
				{
					printf("Engine coolant temperature:%d deg C\r\n", engine_coolant_temperature);
					printf("Engine fuel temperature:%d deg C\r\n", engine_fuel_temperature1);
				}
				else
				{
					printf("Not ready\r\n");
				}
				WAIT_ENTER();
				break;
			case FN_ENGINE_TOTAL_FUEL_USED:
				if(pgn_lfc_recv)
				{
					printf("Engine total fuel used:%dL\r\n", engine_total_fuel_used);
				}
				else
				{
					printf("Not ready\r\n");
				}
				WAIT_ENTER();
				break;
			case FN_ENGINE_SPEED:
				if(pgn_eec1_recv)
				{
					printf("Engine speed:%d RPM\r\n", engine_speed);
				}
				else
				{
					printf("Not ready\r\n");
				}
				WAIT_ENTER();
				break;
			case FN_ACCELERATOR_PEDAL_POSITION:
				if(pgn_eec2_recv)
				{
					printf("Accelerator pedal position:%d%%\r\n", accelerator_pedal_position);
				}
				else
				{
					printf("Not ready\r\n");
				}
				WAIT_ENTER();
				break;
			case FN_VEHICLE_SPEED:
				if(pgn_tco1_recv)
				{
					printf("Vehicle speed:%0.2f km/h\r\n", vehicle_speed);
				}
				else
				{
					printf("Not ready\r\n");
				}
				WAIT_ENTER();
				break;
			case FN_REQUEST_PGN:
				request_pgn();
				break;
			default:
				printf("Unknown choice!\n");
				printf("\nPress ENTER to continue. ");
				WAIT_ENTER();
				break;
		}
	}
	
	j1939_thread_deinit();

	if( (result = vcil_deinit()) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_deinit. error code=0x%08x\n", result);
		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();
		return -1;
	}

	return 0;
}


