#include <stdio.h>

#include "common.h"
#include "vcil/vcil.h"
#include "string.h"

#if defined (__linux__)
#include <pthread.h>
#endif

#include <unistd.h>
static pthread_t can_thread_handle = 0;
static int thread_enable = 0;
static OSEVENT can_rx_event;

char library_version[VCIL_MAXIMUM_LIBRARY_STRING_LENGTH]={0};
char fw_version[VCIL_MAXIMUM_FIRMWARE_VERSION_LENGTH]={0};

bool bit11_mode = 0;

enum {
	FN_SELECT_BITRATE = 1,
	FN_PID_SUPPORTED,
	FN_ENGINE_COOLANT_TEMP,
	FN_ENGINE_RPM,
	FN_VEHICLE_SPEED,
	FN_NUMBER
};

static const char *fn_name[FN_NUMBER-1] = {
	"Select Bitrate & Protocol",
	"Service$1 PID=0x00 check PID 1-32 Supported",
	"Service$1 PID=0x05 Engine Coolant Temperature",
	"Service$1 PID=0x0C Engine RPM",
	"Service$1 PID=0x0D Vehicle Speeed",
};

static const int function_list[FN_NUMBER-1] = {
	FN_SELECT_BITRATE,
	FN_PID_SUPPORTED,
	FN_ENGINE_COOLANT_TEMP,
	FN_ENGINE_RPM,
	FN_VEHICLE_SPEED
};

void show_pid_supported(int s, int base)
{
	for(int i=7;i>=0;i--)
	{
		if( ((s>>i)&0x01) == 0x01)
		{
			printf("PID=%X,", base+(8-i) );
		}
	}
}

void show_obd_info(vcil_can_message_t *msg)
{
	// check response service SID=0x41
	if(msg->data[1] != 0x41)
		return;	
	
	// check SampleOBDTest support showing PID
	switch(msg->data[2])
	{
		// PID Supported
		case 0x00:
		{
			printf("%X Supported:", msg->id);
			show_pid_supported(msg->data[3], 0);
			show_pid_supported(msg->data[4], 8);
			show_pid_supported(msg->data[5], 16);
			show_pid_supported(msg->data[6], 24);
			printf("\n");
		}
		break;
		
		// Engine Coolant Temperature
		case 0x05:
		{
			printf("Engine Coolant Temperature:%d C\n", msg->data[3] - 40);
		}
		break;
		
		// Engine RPM
		case 0x0C:
		{
			unsigned short rpm = ((msg->data[3]<<8) | msg->data[4])/4;
			printf("ECU ID=%X, Engine RPM:%hu\n", msg->id, rpm);
		}
		break;
		
		// Vehicle Speed
		case 0x0D:
		{
			printf("ECU ID=%X, Vehicle Speed: %d km/h\n", msg->id, msg->data[3]);
		}
		break;
	}
}

static void *can_read_thread(void *p)
{
	printf("CAN thread start\n");
	while(thread_enable)
	{
		pthread_mutex_lock(can_rx_event.mutex_handle);
		pthread_cond_wait(can_rx_event.event_handle, can_rx_event.mutex_handle);
		pthread_mutex_unlock(can_rx_event.mutex_handle);

		mrm_err result;
		vcil_can_message_t message;
		while( (result = vcil_can_read(&message)) == MRM_ERR_NO_ERROR)
			show_obd_info(&message);

		if( result != MRM_ERR_VCIL_DATA_NOT_READY )
		{
			printf("vcil_can_read fail. error code=0x%08x\n", result);
			continue;
		}
	
	}
	return 0;
}

static void show_welcome_title(void)
{
	printf("VCIL SDK Sample -- library version:%s\n", library_version);
	printf("                   firmware version:%s\n", fw_version);
	printf(">> Main Menu\n\n");
}

static void show_menu(void)
{
	printf("0) exit\n");
	
	for (int i = 0; i < FN_NUMBER-1; i++)
	{
		printf("%d) %s\n", i + 1, fn_name[i]);		
	}
	
	printf("\nEnter your choice: ");
}

void get_firmware_version()
{
	mrm_err result;
	
	if( (result = vcil_firmware_get_version(fw_version)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_firmware_get_version fail. error code=0x%08x\n", result);
		return;
	}
}

void set_can_speed()
{
	mrm_err result;
	
	vcil_can_speed bitrate;
	
	printf("Speed:\n");
	printf("1) 250K 2.0A\n");
	printf("2) 500K 2.0A\n");
	printf("3) 250K 2.0B\n");
	printf("4) 500K 2.0B\n");
	unsigned int s;
	scanf("%u", &s);
	
	WAIT_ENTER();
	
	if(s == 1 || s== 2)
		bit11_mode = 1;
	
	switch(s)
	{
		case 1:
		case 3:
			bitrate = VCIL_CAN_SPEED_250K;
			break;
		case 2:
		case 4:
			bitrate = VCIL_CAN_SPEED_500K;
			break;
		default:
			printf("Error range\n");
			return;
	}
		
	if( (result = vcil_can_set_speed(0, bitrate)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_can_set_speed fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void request_obd2_service1_pid(int pid)
{
	vcil_can_message_t message;
	
	message.port = 0;	
	message.extended_frame = !bit11_mode;
	message.remote_request = false;
	
	if(message.extended_frame)
		message.id = 0x18DB33F1;
	else	
		message.id = 0x7DF; // External
		
	// OBD2 always DLC = 8
	message.length = 8;
	
	// clear
	memset(&message.data[0], 0, 8);
	
	message.data[0] = 2; 	// payload lenght
	message.data[1] = 1; 	// service $1
	message.data[2] = pid; 	// Request PID

	mrm_err result;

	if( (result = vcil_can_write(&message)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_can_write fail. error code=0x%08x\n", result);
		return;
	}

	//printf("Success\n");
	WAIT_ENTER();
}


void can_thread_init()
{
	pthread_condattr_t cond_attr;
	pthread_condattr_init ( &cond_attr );
	can_rx_event.event_handle = new pthread_cond_t();
	pthread_cond_init ( can_rx_event.event_handle, &cond_attr );
	
	pthread_mutexattr_t attr;
	pthread_mutexattr_init (&attr);
	pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_RECURSIVE);
	can_rx_event.mutex_handle = new pthread_mutex_t();
	pthread_mutex_init (can_rx_event.mutex_handle, &attr);

	mrm_err result;
	
	if( (result = vcil_can_set_event(&can_rx_event)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_can_set_event fail. error code=0x%08x\n", result);
		return;
	}
	
	thread_enable = 1;
	
	pthread_attr_t thread_attr;
	pthread_attr_init(&thread_attr);
	int ret = ::pthread_create(&can_thread_handle, &thread_attr, &can_read_thread, NULL);
	pthread_attr_destroy(&thread_attr);
	
	if( ret < 0)
	{
		printf("can thread create fail.");
		return;
	}
}

void set_module_mode()
{
	mrm_err result;

	if( (result = vcil_set_mode(VCIL_MODE_CAN, VCIL_MODE_CAN, VCIL_MODE_J1708)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_set_mode fail. error code=0x%08x\n", result);
		return;
	}
}


void can_thread_deinit()
{
	thread_enable = 0;
	
	pthread_mutex_lock(can_rx_event.mutex_handle);
	pthread_cond_broadcast(can_rx_event.event_handle);
	pthread_mutex_unlock(can_rx_event.mutex_handle);
}

int main(int argc, char *argv[])
{
	mrm_err result;
	int op;
	
	char path[256] = "/dev/ttyA0";
	if(argc > 1)
	{
		sprintf(path, "%s", argv[1]);
	}	
	printf("Open path:%s\n", path);
	
	if( (result = vcil_init(path)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_init fail. error code=0x%08x\n", result);
		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();
		return -1;
	}
	
	if( (result = vcil_get_version(library_version)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_get_version fail. error code=0x%08x\n", result);
		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();
		return -1;
	}
	
	get_firmware_version();
	
	set_module_mode();
	
	can_thread_init();
	
	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		show_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;

		switch(op)
		{
			case FN_SELECT_BITRATE:
				set_can_speed();
				break;
			case FN_PID_SUPPORTED:
				request_obd2_service1_pid(0x00);
				break;
			case FN_ENGINE_COOLANT_TEMP:
				request_obd2_service1_pid(0x05);
				break;
			case FN_ENGINE_RPM:
				request_obd2_service1_pid(0x0C);
				break;
			case FN_VEHICLE_SPEED:
				request_obd2_service1_pid(0x0D);
				break;
			default:
				printf("Unknown choice!\n");
				printf("\nPress ENTER to continue. ");
				WAIT_ENTER();
				break;
		}
	}
	
	can_thread_deinit();

	if( (result = vcil_deinit()) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_deinit. error code=0x%08x\n", result);
		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();
		return -1;
	}

	return 0;
}


