#include <stdio.h>

#include "common.h"
#include "sdp/library.h"
#include "sdp/backlight.h"

enum backlight_function{
	BACKLIGHT_GET_LEVEL = 1,
	BACKLIGHT_SET_LEVEL,
	BACKLIGHT_GET_BRIGHTNESS,
	BACKLIGHT_SET_BRIGHTNESS,
	BACKLIGHT_GET_LEVEL_RANGE,
	BACKLIGHT_SET_LEVEL_RANGE
};

static unsigned char level_range_max = 0;
static unsigned char level_range_min = 0;

static void show_welcome_title(void)
{
	printf("Smart Display SDK Sample -- library version:%s\n", library_version);
	printf(">> Backlight Menu\n\n");
}

void backlight_menu()
{
	printf("0) back\n");

	printf("%d) Get current backlight level\n", BACKLIGHT_GET_LEVEL);
	printf("%d) Set current backlight level\n", BACKLIGHT_SET_LEVEL);
	printf("%d) Get the level brightness\n", BACKLIGHT_GET_BRIGHTNESS);
	printf("%d) Set the level brightness\n", BACKLIGHT_SET_BRIGHTNESS);
	printf("%d) Get the level range\n", BACKLIGHT_GET_LEVEL_RANGE);
	printf("%d) Set the level range\n", BACKLIGHT_SET_LEVEL_RANGE);
	
	printf("\nEnter your choice: ");
}

int backlight_init()
{
	static int init_once = 0;
	
	if(init_once)
		return 0;
	
	mrm_err result;
	
	if( (result = sdp_backlight_get_max_level_range(&level_range_max, &level_range_min)) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_backlight_get_max_level_range fail. error code=0x%08x\n", result);
		return -1;
	}
	
	init_once = 1;
	
	return 0;
}

void get_current_level()
{
	mrm_err result;

	unsigned char level;

	if( (result = sdp_backlight_get_level(&level)) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_backlight_get_level. error code=0x%08x\n", result);
		return;
	}

	int l = level;
	printf("Current level is %d\n", l);
}

void set_current_level()
{
	mrm_err result;
	int l;
		
	printf("Level (%d-%d):", level_range_min, level_range_max);
	scanf("%d", &l);
	
	WAIT_ENTER();
	
	if( l < level_range_min || l >level_range_max)
	{
		printf("Error range\n");
		return;
	}

	unsigned char level = l;
	if( (result = sdp_backlight_set_level(level)) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_backlight_set_level fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void get_level_brightness()
{
	mrm_err result;
		
	int l;
	printf("Level (%d-%d):", level_range_min, level_range_max);
	scanf("%d", &l);
	
	WAIT_ENTER();
	
	if( l < level_range_min || l >level_range_max)
	{
		printf("Error range\n");
		return;
	}
	
	unsigned char level = l;
	char per;
	
	if( (result = sdp_backlight_get_brightness(level, &per)) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_backlight_get_brightness fail. error code=0x%08x\n", result);
		return;
	}

	int p = per;
	printf("Brightness is %d %%\n", p);
}

void set_level_brightness()
{
	mrm_err result;
	
	int l;
	printf("Level (%d-%d):", level_range_min, level_range_max);
	scanf("%d", &l);
	
	WAIT_ENTER();
	
	if( l < level_range_min || l >level_range_max)
	{
		printf("Error range\n");
		return;
	}
	
	int b;		
	printf("Brightness (0-100):");
	scanf("%d", &b);
	
	WAIT_ENTER();
	
	if( b < 0 || b >100)
	{
		printf("Error range\n");
		return;
	}

	unsigned char level = l;
	char per = b;
	
	if( (result = sdp_backlight_set_brightness(level, per)) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_backlight_set_brightness fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void get_level_range()
{
	mrm_err result;

	unsigned char max;
	unsigned char min;

	if( (result = sdp_backlight_get_level_range(&max, &min)) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_backlight_get_level_range. error code=0x%08x\n", result);
		return;
	}

	int range_max = max;
	int range_min = min;
	printf("Current level range is (%d-%d)\n", range_min, range_max);
}

void set_level_range()
{
	mrm_err result;
	
	int max,min;
		
	printf("Level (%d-%d):\n", level_range_min, level_range_max);
	printf("Min:");
	scanf("%d", &min);
	
	WAIT_ENTER();
	
	if( min < level_range_min || min >level_range_max)
	{
		printf("Error range\n");
		return;
	}
	
	printf("Max:");
	scanf("%d", &max);
	
	WAIT_ENTER();
	
	if( max < level_range_min || max >level_range_max)
	{
		printf("Error range\n");
		return;
	}
	
	if( min>max)
	{
		printf("Error parameter\n");
		return;
	}

	unsigned char min_level = min;
	unsigned char max_level = max;
	if( (result = sdp_backlight_set_level_range(max_level, min_level)) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_backlight_set_level_range fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void backlight_main()
{
	int op;
	
	if(backlight_init()<0)
		return;

	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		backlight_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;
			
		switch(op)
		{
			case BACKLIGHT_GET_LEVEL:
				get_current_level();
				break;
			case BACKLIGHT_SET_LEVEL:
				set_current_level();
				break;
			case BACKLIGHT_GET_BRIGHTNESS:
				get_level_brightness();
				break;
			case BACKLIGHT_SET_BRIGHTNESS:
				set_level_brightness();
				break;
			case BACKLIGHT_GET_LEVEL_RANGE:
				get_level_range();
				break;
			case BACKLIGHT_SET_LEVEL_RANGE:
				set_level_range();
				break;			
			default:
				printf("Unknown choice!\n");
				break;
		}

		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();

	}
}
