#include <stdio.h>

#include "common.h"
#include "vcil/library.h"
#include "vcil/can.h"

#include <string.h>

static int polling_mode = 0;

#if defined (__linux__)
#include <unistd.h>
static pthread_t can_thread_handle = 0;
static int thread_enable = 0;
static OSEVENT can_rx_event;

static void *can_read_thread(void *p)
{
	while(thread_enable)
	{
		if( polling_mode )
		{
			usleep(100000);
		}
		else
		{
			pthread_mutex_lock(can_rx_event.mutex_handle);
			pthread_cond_wait(can_rx_event.event_handle, can_rx_event.mutex_handle);
			pthread_mutex_unlock(can_rx_event.mutex_handle);
		}

		mrm_err result;
		vcil_can_message_t message;
		while( (result = vcil_can_read(&message)) == MRM_ERR_NO_ERROR)
		{
			printf("<CAN,%d>id=%x,", message.port,message.id);
			if(message.remote_request)
			{
				printf("RTR\n");
				continue;
			}
			if(message.extended_frame)
			{
				printf("EX,");
			}
			printf("DLC=%d,", message.length);
			for(int i=0;i<message.length;i++)
				printf("%02x ",message.data[i]);
			printf("\n");			
		}

		if( result != MRM_ERR_VCIL_DATA_NOT_READY )
		{
			printf("vcil_can_read fail. error code=0x%08x\n", result);
			continue;
		}
	
	}
	return 0;
}
#endif

enum page_function{
	CAN_READ_ENABLE = 1,
	CAN_READ_DISABLE,
	CAN_WRITE,
	CAN_SET_SPEED,
	CAN_SET_SPEED_LISTEN_MODE,
	CAN_GET_SPEED,
	CAN_GET_ERROR_STATUS,
	CAN_GET_MASK,
	CAN_SET_MASK,
	CAN_REMOVE_MASK,
	CAN_RESET_MASK,
	CAN_SET_READ_MODE
};

static void show_welcome_title(void)
{
	printf("VCIL SDK Sample -- library version:%s\n", library_version);
	printf(">> CAN Menu Read=%s MODE = %s\n\n", thread_enable ? "Enable" : "Disable",
	polling_mode ? "Polling":"Event");
}

static void page_menu()
{
	printf("0) back\n");

	printf("%d) CAN Read Enable\n", CAN_READ_ENABLE);
	printf("%d) CAN Read Disable\n", CAN_READ_DISABLE);
	printf("%d) CAN Write\n", CAN_WRITE);
	printf("%d) CAN Set Speed\n", CAN_SET_SPEED);
	printf("%d) CAN Set Speed (Listen mode)\n", CAN_SET_SPEED_LISTEN_MODE);
	printf("%d) CAN Get Speed\n", CAN_GET_SPEED);
	printf("%d) CAN Get Error Status\n", CAN_GET_ERROR_STATUS);	
	printf("%d) CAN Get Mask\n", CAN_GET_MASK);
	printf("%d) CAN Set Mask\n", CAN_SET_MASK);
	printf("%d) CAN Remove Mask\n", CAN_REMOVE_MASK);
	printf("%d) CAN Reset Mask\n", CAN_RESET_MASK);
	printf("%d) CAN Set Read Mode\n", CAN_SET_READ_MODE);
		
	printf("\nEnter your choice: ");
}

int can_read_enable()
{
	thread_enable = 1;
	
#if defined (__linux__)
	pthread_attr_t thread_attr;
	pthread_attr_init(&thread_attr);
	int ret = ::pthread_create(&can_thread_handle, &thread_attr, &can_read_thread, NULL);
	pthread_attr_destroy(&thread_attr);
	
	if( ret < 0)
	{
		printf("can thread create fail.");
		return -1;
	}
#endif
	return 0;
}

void can_read_disable()
{
	if(!thread_enable)
		return;
	thread_enable = 0;
	
#if defined (__linux__)
	pthread_mutex_lock(can_rx_event.mutex_handle);
	pthread_cond_broadcast(can_rx_event.event_handle);
	pthread_mutex_unlock(can_rx_event.mutex_handle);
	
	::pthread_join(can_thread_handle, NULL);
	can_thread_handle = 0;
#endif
	return;
}

static int page_init()
{	
	static int init_once = 0;
	
	if(init_once)
		return 0;
				
#if defined (__linux__)
	pthread_condattr_t cond_attr;
	pthread_condattr_init ( &cond_attr );
	can_rx_event.event_handle = new pthread_cond_t();
	pthread_cond_init ( can_rx_event.event_handle, &cond_attr );
	
	pthread_mutexattr_t attr;
	pthread_mutexattr_init (&attr);
	pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_RECURSIVE);
	can_rx_event.mutex_handle = new pthread_mutex_t();
	pthread_mutex_init (can_rx_event.mutex_handle, &attr);
#endif

	mrm_err result;
	
	if( (result = vcil_can_set_event(&can_rx_event)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_can_set_event fail. error code=0x%08x\n", result);
		return -1;
	}
	
	init_once = 1;

	return 0;
}

void can_write()
{
	vcil_can_message_t message;
	
	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}
	message.port = p;
	
	printf("Frame type:\n");
	printf("0) 2.0A\n");
	printf("1) 2.0B\n");
	printf("2) Remote Request 2.0A\n");
	printf("3) Remote Request 2.0B\n");
	unsigned int t;
	scanf("%u", &t);
	
	WAIT_ENTER();
	
	if( t > 3)
	{
		printf("Error range\n");
		return;
	}
	
	if( t %2 == 1)
		message.extended_frame = true;
	else
		message.extended_frame = false;
	
	if(t>1)
		message.remote_request = true;
	else
		message.remote_request = false;
	
	printf("ID (Hex):");
	unsigned int id;
	scanf("%x", &id);
	
	WAIT_ENTER();
	
	if( (t % 2 == 0 && id > 0x7FF) ||
	(t %2 == 1 && id > 0x1FFFFFFF) )
	{
		printf("Error range\n");
		return;
	}
	
	message.id = id;
	
	printf("DLC (Max. 8):");
	unsigned int dlc;
	scanf("%u", &dlc);
	
	WAIT_ENTER();
	
	if( dlc > 8)
	{
		printf("Error range\n");
		return;
	}
	
	message.length = dlc;
	
	for(unsigned int i = 0;i<dlc;i++)
	{
		printf("Data[%u] (Hex):", i);
		unsigned int data;
		scanf("%x", &data);
	
		WAIT_ENTER();
	
		if( data > 0xFF)
		{
			printf("Error range\n");
			return;
		}
		message.data[i] = data;
	}
	
	mrm_err result;

	if( (result = vcil_can_write(&message)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_can_write fail. error code=0x%08x\n", result);
		return;
	}


	printf("Success\n");
}

void set_can_speed(int listen_only)
{
	mrm_err result;
	
	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}

	vcil_can_speed bitrate;
	
	printf("Speed:\n");
	printf("0) 125K\n");
	printf("1) 250K\n");
	printf("2) 500K\n");
	printf("3) 1M\n");	
	printf("4) 200K\n");
	printf("5) 100K\n");
	printf("6) 800K\n");
	printf("7) 83.33K\n");
	printf("8) 50K\n");
	printf("9) 20K\n");
	printf("10) 10K\n");
	printf("11) 5K\n");	
		
	unsigned int s;
	scanf("%u", &s);
	
	WAIT_ENTER();
	
	switch(s)
	{
		case 0:
			bitrate = VCIL_CAN_SPEED_125K;
			break;
		case 1:
			bitrate = VCIL_CAN_SPEED_250K;
			break;
		case 2:
			bitrate = VCIL_CAN_SPEED_500K;
			break;
		case 3:
			bitrate = VCIL_CAN_SPEED_1M;
			break;			
		case 4:
			bitrate = VCIL_CAN_SPEED_200K;
			break;
		case 5:
			bitrate = VCIL_CAN_SPEED_100K;
			break;
		case 6:
			bitrate = VCIL_CAN_SPEED_800K;
			break;
		case 7:
			bitrate = VCIL_CAN_SPEED_83K;
			break;
		case 8:
			bitrate = VCIL_CAN_SPEED_50K;
			break;
		case 9:
			bitrate = VCIL_CAN_SPEED_20K;
			break;
		case 10:
			bitrate = VCIL_CAN_SPEED_10K;
			break;
		case 11:
			bitrate = VCIL_CAN_SPEED_5K;
			break;
		default:
			printf("Error range\n");
			return;
	}
	
	if( listen_only != 0 )
	{
		if( (result = vcil_can_set_speed(p, bitrate)) != MRM_ERR_NO_ERROR )
		{
			printf("vcil_can_set_speed fail. error code=0x%08x\n", result);
			return;
		}
	}
	else
	{
		if( (result = vcil_can_set_speed_listen_mode(p, bitrate)) != MRM_ERR_NO_ERROR )
		{
			printf("vcil_can_set_speed_listen_mode fail. error code=0x%08x\n", result);
			return;
		}
	}
	
	printf("Success\n");
}

void get_can_speed()
{
	mrm_err result;
	
	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}
	
	vcil_can_speed bitrate;
	vcil_can_bus_mode mode;
			
	if( (result = vcil_can_get_speed(p, &bitrate, &mode)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_can_get_speed fail. error code=0x%08x\n", result);
		return;
	}
	
	switch(mode)
	{
		case VCIL_CAN_BUS_NORMAL_MODE:
			printf("Normal mode ");
			break;
		case VCIL_CAN_BUS_LISTEN_MODE:
			printf("Listen mode ");
			break;
		case VCIL_CAN_BUS_INIT_MODE:
			printf("CAN bus not initialization");
			return;
		default:
			break;
	}
		
	switch(bitrate)
	{
		case VCIL_CAN_SPEED_125K:
			printf("125K\n");
			break;
		case VCIL_CAN_SPEED_250K:
			printf("250K\n");
			break;
		case VCIL_CAN_SPEED_500K:
			printf("500K\n");
			break;
		case VCIL_CAN_SPEED_1M:
			printf("1M\n");
			break;			
		case VCIL_CAN_SPEED_200K:
			printf("200K\n");
			break;
		case VCIL_CAN_SPEED_100K:
			printf("100K\n");
			break;
		case VCIL_CAN_SPEED_800K:
			printf("800K\n");
			break;
		case VCIL_CAN_SPEED_83K:
			printf("83.33K\n");
			break;
		case VCIL_CAN_SPEED_50K:
			printf("50K\n");
			break;
		case VCIL_CAN_SPEED_20K:
			printf("20K\n");
			break;
		case VCIL_CAN_SPEED_10K:
			printf("10K\n");
			break;
		case VCIL_CAN_SPEED_5K:
			printf("5K\n");
			break;
		case VCIL_CAN_SPEED_USER_DEFINE:
			printf("User define, some bitrate may user define by library. The firmware not native support");
			break;
		default:
			printf("Error range\n");
			return;
	}
}

void get_can_error_status()
{
	mrm_err result;
	
	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}
	
	vcil_can_error_status es;
			
	if( (result = vcil_can_get_bus_error_status(p, &es)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_can_get_bus_error_status fail. error code=0x%08x\n", result);
		return;
	}
		
	printf("Receive error counter:%u\n", es.rec);
	printf("Transmit error counter:%u\n", es.tec);
	
	switch(es.last_error_code)
	{
		case 0:
			printf("No error\n");
			break;
		case 1:
			printf("Stuff Error\n");
			break;
		case 2:
			printf("Form Error\n");
			break;
		case 3:
			printf("Acknowledgment Error\n");
			break;
		case 4:
			printf("Bit recessive Error\n");
			break;
		case 5:
			printf("Bit dominant Error\n");
			break;
		case 6:
			printf("CRC Error\n");
			break;
		case 7:
			printf("Set by software\n");
			break;			
	}
	
	if( es.error_flag & 0x01)
		printf("Error warning flag\n");
		
	if( es.error_flag & 0x02)
		printf("Error passive flag\n");
		
	if( es.error_flag & 0x04)
		printf("Bus-off flag\n");
}

void get_can_mask()
{
	mrm_err result;
	
	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}
	
	printf("Bank (0-13):");
	unsigned int b;
	scanf("%u", &b);
	
	WAIT_ENTER();
	
	if( b > 13)
	{
		printf("Error range\n");
		return;
	}
	
	vcil_can_mask_t mask;
	mask.bank = b;
		
	if( (result = vcil_can_get_mask(p, &mask)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_can_get_mask fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Mask:\n");
	printf("Mask Type:%d\n", mask.type);
	if(mask.extended_frame)
		printf("Frame Type:2.0B");
	else
		printf("Frame Type:2.0A");
	if(mask.remote_request)
		printf("+RTR\n");
	else
		printf("\n");
	printf("ID(Hex):%x\n", mask.id1);
	printf("MASK(Hex):%x\n", mask.mask1);	
}

void set_can_mask()
{
	mrm_err result;
	vcil_can_mask_t mask;
	memset(&mask, 0, sizeof(vcil_can_mask_t));
	
	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}
	
	printf("Bank (0-13):");
	unsigned int b;
	scanf("%u", &b);
	
	WAIT_ENTER();
	
	if( b > 13)
	{
		printf("Error range\n");
		return;
	}
	
	printf("Frame type:\n");
	printf("0) 2.0A\n");
	printf("1) 2.0B\n");
	printf("2) Remote Request 2.0A\n");
	printf("3) Remote Request 2.0B\n");
	unsigned int t;
	scanf("%u", &t);
	
	WAIT_ENTER();
	
	if( t > 3)
	{
		printf("Error range\n");
		return;
	}
	
	if( t %2 == 1)
		mask.extended_frame = true;
	else
		mask.extended_frame = false;
	
	if(t>1)
		mask.remote_request = true;
	else
		mask.remote_request = false;
	
	printf("ID (Hex):");
	unsigned int id;
	scanf("%x", &id);
	
	WAIT_ENTER();
	
	if( (t % 2 == 0 && id > 0x7FF) ||
	(t %2 == 1 && id > 0x1FFFFFFF) )
	{
		printf("Error range\n");
		return;
	}
	mask.id1 = id;
	
	printf("Mask (Hex):");
	unsigned int m;
	scanf("%x", &m);
	
	WAIT_ENTER();
	
	if( (t % 2 == 0 && m > 0x7FF) ||
	(t %2 == 1 && m > 0x1FFFFFFF) )
	{
		printf("Error range\n");
		return;
	}
	mask.mask1 = m;
	
	mask.bank = b;
		
	if( (result = vcil_can_set_mask(p, &mask)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_can_set_mask fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void remove_can_mask()
{
	mrm_err result;

	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}
	
	printf("Bank (0-13):");
	unsigned int b;
	scanf("%u", &b);
	
	WAIT_ENTER();
	
	if( b > 13)
	{
		printf("Error range\n");
		return;
	}
			
	if( (result = vcil_can_remove_mask(p, b)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_can_remove_mask fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void reset_can_mask()
{
	mrm_err result;
	
	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}
	
	if( (result = vcil_can_reset_mask(p)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_can_reset_mask fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

static void set_read_mode()
{
	printf("Mode:\n");
	printf("0) Event trigger\n");
	printf("1) Polling\n");
	unsigned int m;
	scanf("%u", &m);
	
	WAIT_ENTER();
	
	if( m > 1)
	{
		printf("Error range\n");
		return;
	}
	
	polling_mode = m;
	
#if defined (__linux__)
	pthread_mutex_lock(can_rx_event.mutex_handle);
	pthread_cond_broadcast(can_rx_event.event_handle);
	pthread_mutex_unlock(can_rx_event.mutex_handle);
#endif

	printf("Success\n");
}

void can_main()
{
	int op;
	
	if( page_init() < 0 )
		return ;
		
	if( can_read_enable() < 0)
		return;

	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		page_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;
			
		switch(op)
		{
			case CAN_READ_ENABLE:
				can_read_enable();
				break;
			case CAN_READ_DISABLE:
				can_read_disable();
				break;
			case CAN_WRITE:
				can_write();
				break;
			case CAN_SET_SPEED:
				set_can_speed(1);
				break;
			case CAN_SET_SPEED_LISTEN_MODE:
				set_can_speed(0);
				break;
			case CAN_GET_SPEED:
				get_can_speed();
				break;
			case CAN_GET_ERROR_STATUS:
				get_can_error_status();
				break;
			case CAN_GET_MASK:
				get_can_mask();
				break;
			case CAN_SET_MASK:
				set_can_mask();
				break;
			case CAN_REMOVE_MASK:
				remove_can_mask();
				break;
			case CAN_RESET_MASK:
				reset_can_mask();
				break;
			case CAN_SET_READ_MODE:
				set_read_mode();
				break;
			default:
				printf("Unknown choice!\n");
				break;
		}

		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();

	}
	
	can_read_disable();	
}
