#include <stdio.h>

#include "common.h"
#include "vcil/library.h"
#include "vcil/j1939.h"

static int polling_mode = 0;

#if defined (__linux__)
#include <unistd.h>
static pthread_t j1939_thread_handle = 0;
static int thread_enable = 0;
static OSEVENT j1939_rx_event;

static void *j1939_read_thread(void *p)
{
	while(thread_enable)
	{
		if( polling_mode )
		{
			usleep(100000);
		}
		else
		{
			pthread_mutex_lock(j1939_rx_event.mutex_handle);
			pthread_cond_wait(j1939_rx_event.event_handle, j1939_rx_event.mutex_handle);
			pthread_mutex_unlock(j1939_rx_event.mutex_handle);
		}

		mrm_err result;
		vcil_j1939_message_t message;	
		while( (result = vcil_j1939_read(&message)) == MRM_ERR_NO_ERROR)
		{
			printf("<J1939,%d>p=%d,pgn=%x,dst=%x,src=%x,",
			message.port,
			message.priority,
			message.pgn,
			message.destination,
			message.source);
			printf("DLC=%d,", message.length);
			for(int i=0;i<message.length;i++)
				printf("%02x ",message.data[i]);
			printf("\n");			
		}

		if( result != MRM_ERR_VCIL_DATA_NOT_READY )
		{
			printf("vcil_j1939_read fail. error code=0x%08x\n", result);
			continue;
		}
	
	}
	return 0;
}
#endif

enum page_function{
	J1939_READ_ENABLE = 1,
	J1939_READ_DISABLE,
	J1939_WRITE,
	J1939_GET_CONFIG,
	J1939_SET_CONFIG,
	J1939_GET_MASK,
	J1939_ADD_MASK,
	J1939_REMOVE_MASK,
	J1939_RESET_MASK,
	J1939_SET_READ_MODE
};

static void show_welcome_title(void)
{
	printf("VCIL SDK Sample -- library version:%s\n", library_version);
	printf(">> J1939 Menu Read=%s MODE = %s\n\n", thread_enable ? "Enable" : "Disable",
	polling_mode ? "Polling": "Event");
}

static void page_menu()
{
	printf("0) back\n");

	printf("%d) J1939 Read Enable\n", J1939_READ_ENABLE);
	printf("%d) J1939 Read Disable\n", J1939_READ_DISABLE);
	printf("%d) J1939 Write\n", J1939_WRITE);
	printf("%d) J1939 Get Config\n", J1939_GET_CONFIG);
	printf("%d) J1939 Set Config\n", J1939_SET_CONFIG);
	printf("%d) J1939 Get Mask\n", J1939_GET_MASK);
	printf("%d) J1939 Add Mask\n", J1939_ADD_MASK);
	printf("%d) J1939 Remove Mask\n", J1939_REMOVE_MASK);
	printf("%d) J1939 Reset Mask\n", J1939_RESET_MASK);
	printf("%d) J1939 Set Read Mode\n", J1939_SET_READ_MODE);
		
	printf("\nEnter your choice: ");
}

int j1939_read_enable()
{
	thread_enable = 1;
	
#if defined (__linux__)
	pthread_attr_t thread_attr;
	pthread_attr_init(&thread_attr);
	int ret = ::pthread_create(&j1939_thread_handle, &thread_attr, &j1939_read_thread, NULL);
	pthread_attr_destroy(&thread_attr);
	
	if( ret < 0)
	{
		printf("j1939 thread create fail.");
		return -1;
	}
#endif
	return 0;
}

void j1939_read_disable()
{
	if(!thread_enable)
		return;
	thread_enable = 0;
	
#if defined (__linux__)
	pthread_mutex_lock(j1939_rx_event.mutex_handle);
	pthread_cond_broadcast(j1939_rx_event.event_handle);
	pthread_mutex_unlock(j1939_rx_event.mutex_handle);
	
	::pthread_join(j1939_thread_handle, NULL);
	j1939_thread_handle = 0;
#endif
	return;
}

static int page_init()
{	
	static int init_once = 0;
	
	if(init_once)
		return 0;
				
#if defined (__linux__)
	pthread_condattr_t cond_attr;
	pthread_condattr_init ( &cond_attr );
	j1939_rx_event.event_handle = new pthread_cond_t();
	pthread_cond_init ( j1939_rx_event.event_handle, &cond_attr );
	
	pthread_mutexattr_t attr;
	pthread_mutexattr_init (&attr);
	pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_RECURSIVE);
	j1939_rx_event.mutex_handle = new pthread_mutex_t();
	pthread_mutex_init (j1939_rx_event.mutex_handle, &attr);
#endif

	mrm_err result;
	
	if( (result = vcil_j1939_set_event(&j1939_rx_event)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_j1939_set_event fail. error code=0x%08x\n", result);
		return -1;
	}
	
	init_once = 1;

	return 0;
}

void j1939_write()
{
	vcil_j1939_message_t message;
	
	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}
	message.port = p;
	
	printf("Priority (0-7):");
	unsigned int pri;
	scanf("%u", &pri);
	
	WAIT_ENTER();
	
	if( pri > 7 )
	{
		printf("Error range\n");
		return;
	}
	
	message.priority = pri;
		
	printf("PGN (Hex):");
	unsigned int pgn;
	scanf("%x", &pgn);
	
	WAIT_ENTER();
	
	if( pgn > 0x1FFFF )
	{
		printf("Error range\n");
		return;
	}
	
	message.pgn = pgn;
	
	printf("DST (Hex 00-FF) <Note FF=boardcast address FE=null address>:");
	unsigned int dst;
	scanf("%x", &dst);
	
	WAIT_ENTER();
	
	if( dst > 0xFF )
	{
		printf("Error range\n");
		return;
	}
	
	message.destination = dst;
	
	printf("SRC (Hex 00-FD):" );
	unsigned int src;
	scanf("%x", &src);
	
	WAIT_ENTER();
	
	if( src > 0xFF )
	{
		printf("Error range\n");
		return;
	}
	
	message.source = src;
	
	printf("DLC (Max. buffer %d):", VCIL_MAX_J1939_DATA_SIZE);
	unsigned int dlc;
	scanf("%u", &dlc);
	
	WAIT_ENTER();
	
	if( dlc > VCIL_MAX_J1939_DATA_SIZE)
	{
		printf("Error range\n");
		return;
	}
	
	message.length = dlc;
	
	for(unsigned int i = 0;i<dlc;i++)
	{
		printf("Data[%u] (Hex):", i);
		unsigned int data;
		scanf("%x", &data);
	
		WAIT_ENTER();
	
		if( data > 0xFF)
		{
			printf("Error range\n");
			return;
		}
		message.data[i] = data;
	}
	
	mrm_err result;

	if( (result = vcil_j1939_write(&message)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_j1939_write fail. error code=0x%08x\n", result);
		return;
	}


	printf("Success\n");
}

void get_j1939_mask()
{
	mrm_err result;
	
	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}

	unsigned int total_mask = 0;
	if( (result = vcil_j1939_get_mask_number(p, &total_mask)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_j1939_get_mask_number fail. error code=0x%08x\n", result);
		return;
	}
	
	if(total_mask == 0)
	{
		printf("There no mask exist.\n");
		return;
	}
	
	unsigned int mask_pgn[128];
	
	if( (result = vcil_j1939_get_all_mask(p, mask_pgn)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_j1939_get_all_mask fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Mask PGN:\n");
	for(unsigned int i = 0; i<total_mask;i++)
		printf("%x,",mask_pgn[i]);
	printf("\b \n");	
}

void add_j1939_mask()
{
	mrm_err result;
	
	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}
		
	printf("PGN (Hex):");
	unsigned int pgn;
	scanf("%x", &pgn);
	
	WAIT_ENTER();
	
	if( pgn > 0x1FFFF )
	{
		printf("Error range\n");
		return;
	}
			
	if( (result = vcil_j1939_add_mask(p, pgn)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_j1939_add_mask fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void remove_j1939_mask()
{
	mrm_err result;

	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}
	
	printf("PGN (Hex):");
	unsigned int pgn;
	scanf("%x", &pgn);
	
	WAIT_ENTER();
	
	if( pgn > 0x1FFFF )
	{
		printf("Error range\n");
		return;
	}
			
	if( (result = vcil_j1939_remove_mask(p, pgn)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_j1939_remove_mask fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void reset_j1939_mask()
{
	mrm_err result;
	
	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}
	
	if( (result = vcil_j1939_remove_all_mask(p)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_j1939_remove_all_mask fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

static void set_read_mode()
{
	printf("Mode:\n");
	printf("0) Event trigger\n");
	printf("1) Polling\n");
	unsigned int m;
	scanf("%u", &m);
	
	WAIT_ENTER();
	
	if( m > 1)
	{
		printf("Error range\n");
		return;
	}
	
	polling_mode = m;
	
#if defined (__linux__)
	pthread_mutex_lock(j1939_rx_event.mutex_handle);
	pthread_cond_broadcast(j1939_rx_event.event_handle);
	pthread_mutex_unlock(j1939_rx_event.mutex_handle);
#endif

	printf("Success\n");
}

void get_j1939_config()
{
	mrm_err result;
	
	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}
		
	vcil_j1939_config_t config;
	if( (result = vcil_j1939_get_config(p, &config)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_j1939_get_config. error code=0x%08x\n", result);
		return;
	}

	printf("J1939 Port%d Config:\n", p);
	printf("Address:%x\n", config.address);
	printf("Arbitrary Address Capable:%d\n", config.arbitrary_address_capable);
	printf("Industry Group:%d\n", config.industry_group);
	printf("Vehicle System Instance:%d\n", config.vehicle_system_instance);
	printf("Vehicle System:%d\n", config.vehicle_system);
	printf("Function:%d\n", config.function);
	printf("Function Instance:%d\n", config.function_instance);
	printf("ECU Instance:%d\n", config.ecu_instance);
	printf("Manufacturer Code:%hd\n", config.manufacturer_code);
	printf("Identity Number:%x\n", config.identity_number);	
}

void set_j1939_config()
{
	mrm_err result;
	
	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}
		
	vcil_j1939_config_t config;
	
	printf("Address (Hex 00-FD):");
	unsigned int address;
	scanf("%x", &address);
	
	WAIT_ENTER();
	
	if(address > 0xFF)
	{
		printf("Error range\n");
		return;
	}
	
	config.address = address;
	
	printf("Arbitrary Address Capable:");
	unsigned int arbitrary_address_capable;
	scanf("%u", &arbitrary_address_capable);
	
	WAIT_ENTER();
	config.arbitrary_address_capable = arbitrary_address_capable;
	
	printf("Industry Group:");
	unsigned int industry_group;
	scanf("%u", &industry_group);
	
	WAIT_ENTER();
	config.industry_group = industry_group;
	
	printf("Vehicle System Instance:");
	unsigned int vehicle_system_instance;
	scanf("%u", &vehicle_system_instance);
	
	WAIT_ENTER();
	config.vehicle_system_instance = vehicle_system_instance;
	
	printf("Vehicle System:");
	unsigned int vehicle_system;
	scanf("%u", &vehicle_system);
	
	WAIT_ENTER();
	config.vehicle_system = vehicle_system;
	
	printf("Function:");
	unsigned int function;
	scanf("%u", &function);
	
	WAIT_ENTER();
	config.function = function;
	
	printf("Function Instance:");
	unsigned int function_instance;
	scanf("%u", &function_instance);
	
	WAIT_ENTER();
	config.function_instance = function_instance;
	
	printf("ECU Instance:");
	unsigned int ecu_instance;
	scanf("%u", &ecu_instance);
	
	WAIT_ENTER();
	config.ecu_instance = ecu_instance;
	
	printf("Manufacturer Code:");
	unsigned int manufacturer_code;
	scanf("%u", &manufacturer_code);
	
	WAIT_ENTER();
	config.manufacturer_code = manufacturer_code;
	
	printf("Identity Number(Hex):");
	unsigned int identity_number;
	scanf("%x", &identity_number);
	printf("Identity Number(Hex) debug:%x\n", identity_number);
	
	WAIT_ENTER();
	config.identity_number = identity_number;
	
	if( (result = vcil_j1939_set_config(p, &config)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_j1939_set_config. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void j1939_main()
{
	int op;
	
	if( page_init() < 0 )
		return ;
		
	if( j1939_read_enable() < 0)
		return;

	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		page_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;
			
		switch(op)
		{
			case J1939_READ_ENABLE:
				j1939_read_enable();
				break;
			case J1939_READ_DISABLE:
				j1939_read_disable();
				break;
			case J1939_WRITE:
				j1939_write();
				break;
			case J1939_GET_CONFIG:
				get_j1939_config();
				break;
			case J1939_SET_CONFIG:
				set_j1939_config();
				break;
			case J1939_GET_MASK:
				get_j1939_mask();
				break;
			case J1939_ADD_MASK:
				add_j1939_mask();
				break;
			case J1939_REMOVE_MASK:
				remove_j1939_mask();
				break;
			case J1939_RESET_MASK:
				reset_j1939_mask();
				break;
			case J1939_SET_READ_MODE:
				set_read_mode();
				break;
			default:
				printf("Unknown choice!\n");
				break;
		}

		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();

	}
	
	j1939_read_disable();	
}
