#include <stdio.h>

#include "common.h"
#include "vcil/library.h"
#include "vcil/obd2.h"

static int polling_mode = 0;

#if defined (__linux__)
#include <unistd.h>
static pthread_t obd2_thread_handle = 0;
static int thread_enable = 0;
static OSEVENT obd2_rx_event;

static void *obd2_read_thread(void *p)
{
	while(thread_enable)
	{
		if( polling_mode )
		{
			usleep(100000);
		}
		else
		{
			pthread_mutex_lock(obd2_rx_event.mutex_handle);
			pthread_cond_wait(obd2_rx_event.event_handle, obd2_rx_event.mutex_handle);
			pthread_mutex_unlock(obd2_rx_event.mutex_handle);
		}

		mrm_err result;
		vcil_obd2_message_t message;	
		while( (result = vcil_obd2_read(&message)) == MRM_ERR_NO_ERROR)
		{
			printf("<OBD2,%d>p=%d,%s,dst=%x,src=%x,",
			message.port,
			message.priority,
			message.type == VCIL_OBD2_TYPE_PHYSICAL ? "PHY" : "FUN",			
			message.destination,
			message.source);
			printf("DLC=%d,", message.length);
			for(int i=0;i<message.length;i++)
				printf("%02x ",message.data[i]);
			printf("\n");			
		}

		if( result != MRM_ERR_VCIL_DATA_NOT_READY )
		{
			printf("vcil_obd2_read fail. error code=0x%08x\n", result);
			continue;
		}
	
	}
	return 0;
}
#endif

enum page_function{
	OBD2_READ_ENABLE = 1,
	OBD2_READ_DISABLE,
	OBD2_WRITE,
	OBD2_GET_MASK,
	OBD2_ADD_MASK,
	OBD2_REMOVE_MASK,
	OBD2_RESET_MASK,
	OBD2_SET_READ_MODE
};

static void show_welcome_title(void)
{
	printf("VCIL SDK Sample -- library version:%s\n", library_version);
	printf(">> OBD2 Menu Read=%s MODE = %s\n\n", thread_enable ? "Enable" : "Disable",
	polling_mode ? "Polling": "Event");
}

static void page_menu()
{
	printf("0) back\n");

	printf("%d) OBD2 Read Enable\n", OBD2_READ_ENABLE);
	printf("%d) OBD2 Read Disable\n", OBD2_READ_DISABLE);
	printf("%d) OBD2 Write\n", OBD2_WRITE);
	printf("%d) OBD2 Get Mask\n", OBD2_GET_MASK);
	printf("%d) OBD2 Add Mask\n", OBD2_ADD_MASK);
	printf("%d) OBD2 Remove Mask\n", OBD2_REMOVE_MASK);
	printf("%d) OBD2 Reset Mask\n", OBD2_RESET_MASK);
	printf("%d) OBD2 Set Read Mode\n", OBD2_SET_READ_MODE);
		
	printf("\nEnter your choice: ");
}

int obd2_read_enable()
{
	thread_enable = 1;
	
#if defined (__linux__)
	pthread_attr_t thread_attr;
	pthread_attr_init(&thread_attr);
	int ret = ::pthread_create(&obd2_thread_handle, &thread_attr, &obd2_read_thread, NULL);
	pthread_attr_destroy(&thread_attr);
	
	if( ret < 0)
	{
		printf("obd2 thread create fail.");
		return -1;
	}
#endif
	return 0;
}

void obd2_read_disable()
{
	if(!thread_enable)
		return;
	thread_enable = 0;
	
#if defined (__linux__)
	pthread_mutex_lock(obd2_rx_event.mutex_handle);
	pthread_cond_broadcast(obd2_rx_event.event_handle);
	pthread_mutex_unlock(obd2_rx_event.mutex_handle);
	
	::pthread_join(obd2_thread_handle, NULL);
	obd2_thread_handle = 0;
#endif
	return;
}

static int page_init()
{	
	static int init_once = 0;
	
	if(init_once)
		return 0;
				
#if defined (__linux__)
	pthread_condattr_t cond_attr;
	pthread_condattr_init ( &cond_attr );
	obd2_rx_event.event_handle = new pthread_cond_t();
	pthread_cond_init ( obd2_rx_event.event_handle, &cond_attr );
	
	pthread_mutexattr_t attr;
	pthread_mutexattr_init (&attr);
	pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_RECURSIVE);
	obd2_rx_event.mutex_handle = new pthread_mutex_t();
	pthread_mutex_init (obd2_rx_event.mutex_handle, &attr);
#endif

	mrm_err result;
	
	if( (result = vcil_obd2_set_event(&obd2_rx_event)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_obd2_set_event fail. error code=0x%08x\n", result);
		return -1;
	}
	
	init_once = 1;

	return 0;
}

void obd2_write()
{
	vcil_obd2_message_t message;
	
	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}
	message.port = p;
	
	printf("Priority (0-7):");
	unsigned int pri;
	scanf("%u", &pri);
	
	WAIT_ENTER();
	
	if( pri > 7 )
	{
		printf("Error range\n");
		return;
	}
	
	message.priority = pri;
		
	printf("Type:\n");
	printf("0) PHYSICAL:\n");
	printf("1) FUNCTIONAL:\n");
	unsigned int t;
	scanf("%u", &t);
	
	WAIT_ENTER();
	
	if( t > 1 )
	{
		printf("Error range\n");
		return;
	}
	
	message.type = t == 0 ? VCIL_OBD2_TYPE_PHYSICAL : VCIL_OBD2_TAPE_FUNCTIONAL;
	
	printf("DST (Hex 00-FF):");
	unsigned int dst;
	scanf("%x", &dst);
	
	WAIT_ENTER();
	
	if( dst > 0xFF )
	{
		printf("Error range\n");
		return;
	}
	
	message.destination = dst;
	
	printf("SRC (Hex 00-FF):" );
	unsigned int src;
	scanf("%x", &src);
	
	WAIT_ENTER();
	
	if( src > 0xFF )
	{
		printf("Error range\n");
		return;
	}
	
	message.source = src;
	
	printf("DLC (Max. buffer %d):", VCIL_MAX_OBD2_DATA_SIZE);
	unsigned int dlc;
	scanf("%u", &dlc);
	
	WAIT_ENTER();
	
	if( dlc > VCIL_MAX_OBD2_DATA_SIZE)
	{
		printf("Error range\n");
		return;
	}
	
	message.length = dlc;
	
	for(unsigned int i = 0;i<dlc;i++)
	{
		printf("Data[%u] (Hex):", i);
		unsigned int data;
		scanf("%x", &data);
	
		WAIT_ENTER();
	
		if( data > 0xFF)
		{
			printf("Error range\n");
			return;
		}
		message.data[i] = data;
	}
	
	mrm_err result;

	if( (result = vcil_obd2_write(&message)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_obd2_write fail. error code=0x%08x\n", result);
		return;
	}


	printf("Success\n");
}

void get_obd2_mask()
{
	mrm_err result;
	
	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}

	unsigned int total_mask = 0;
	if( (result = vcil_obd2_get_mask_number(p, &total_mask)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_obd2_get_mask_number fail. error code=0x%08x\n", result);
		return;
	}
	
	if(total_mask == 0)
	{
		printf("There no mask exist.\n");
		return;
	}
	
	unsigned int mask_pid[128];
	
	if( (result = vcil_obd2_get_all_mask(p, mask_pid)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_obd2_get_all_mask fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Mask PID:\n");
	for(unsigned int i = 0; i<total_mask;i++)
		printf("%x,",mask_pid[i]);
	printf("\b \n");	
}

void add_obd2_mask()
{
	mrm_err result;
	
	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}
		
	printf("PID (Hex 00-FF):");
	unsigned int pid;
	scanf("%x", &pid);
	
	WAIT_ENTER();
	
	if( pid > 0xFF )
	{
		printf("Error range\n");
		return;
	}
			
	if( (result = vcil_obd2_add_mask(p, pid)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_obd2_add_mask fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void remove_obd2_mask()
{
	mrm_err result;

	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}
	
	printf("PID (Hex 00-FF):");
	unsigned int pid;
	scanf("%x", &pid);
	
	WAIT_ENTER();
	
	if( pid > 0xFF )
	{
		printf("Error range\n");
		return;
	}
			
	if( (result = vcil_obd2_remove_mask(p, pid)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_obd2_remove_mask fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void reset_obd2_mask()
{
	mrm_err result;
	
	printf("Port (0-1):");
	unsigned int p;
	scanf("%u", &p);
	
	WAIT_ENTER();
	
	if( p > 1)
	{
		printf("Error range\n");
		return;
	}
	
	if( (result = vcil_obd2_remove_all_mask(p)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_obd2_remove_all_mask fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

static void set_read_mode()
{
	printf("Mode:\n");
	printf("0) Event trigger\n");
	printf("1) Polling\n");
	unsigned int m;
	scanf("%u", &m);
	
	WAIT_ENTER();
	
	if( m > 1)
	{
		printf("Error range\n");
		return;
	}
	
	polling_mode = m;
	
#if defined (__linux__)
	pthread_mutex_lock(obd2_rx_event.mutex_handle);
	pthread_cond_broadcast(obd2_rx_event.event_handle);
	pthread_mutex_unlock(obd2_rx_event.mutex_handle);
#endif

	printf("Success\n");
}

void obd2_main()
{
	int op;
	
	if( page_init() < 0 )
		return ;
		
	if( obd2_read_enable() < 0)
		return;

	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		page_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;
			
		switch(op)
		{
			case OBD2_READ_ENABLE:
				obd2_read_enable();
				break;
			case OBD2_READ_DISABLE:
				obd2_read_disable();
				break;
			case OBD2_WRITE:
				obd2_write();
				break;
			case OBD2_GET_MASK:
				get_obd2_mask();
				break;
			case OBD2_ADD_MASK:
				add_obd2_mask();
				break;
			case OBD2_REMOVE_MASK:
				remove_obd2_mask();
				break;
			case OBD2_RESET_MASK:
				reset_obd2_mask();
				break;
			case OBD2_SET_READ_MODE:
				set_read_mode();
				break;
			default:
				printf("Unknown choice!\n");
				break;
		}

		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();

	}
	
	obd2_read_disable();	
}
